/**
 * data-structure-typed
 *
 * @author Tyler Zeng
 * @copyright Copyright (c) 2022 Tyler Zeng <zrwusa@gmail.com>
 * @license MIT License
 */
import { DoublyLinkedList } from '../linked-list';
export declare class Deque<E = any> extends DoublyLinkedList<E> {
}
export declare class ObjectDeque<E = number> {
    constructor(capacity?: number);
    protected _nodes: {
        [key: number]: E;
    };
    get nodes(): {
        [p: number]: E;
    };
    protected _capacity: number;
    get capacity(): number;
    protected _first: number;
    get first(): number;
    protected _last: number;
    get last(): number;
    protected _size: number;
    get size(): number;
    /**
     * Time Complexity: O(1)
     * Space Complexity: O(1)
     */
    /**
     * Time Complexity: O(1)
     * Space Complexity: O(1)
     *
     * The "addFirst" function adds a value to the beginning of an array-like data structure.
     * @param {E} value - The `value` parameter represents the value that you want to add to the beginning of the data
     * structure.
     */
    addFirst(value: E): void;
    /**
     * Time Complexity: O(1)
     * Space Complexity: O(1)
     */
    /**
     * Time Complexity: O(1)
     * Space Complexity: O(1)
     *
     * The addLast function adds a value to the end of an array-like data structure.
     * @param {E} value - The `value` parameter represents the value that you want to add to the end of the data structure.
     */
    addLast(value: E): void;
    /**
     * Time Complexity: O(1)
     * Space Complexity: O(1)
     */
    /**
     * Time Complexity: O(1)
     * Space Complexity: O(1)
     *
     * The function `popFirst()` removes and returns the first element in a data structure.
     * @returns The value of the first element in the data structure.
     */
    popFirst(): E | undefined;
    /**
     * Time Complexity: O(1)
     * Space Complexity: O(1)
     */
    /**
     * Time Complexity: O(1)
     * Space Complexity: O(1)
     *
     * The `getFirst` function returns the first element in an array-like data structure if it exists.
     * @returns The element at the first position of the `_nodes` array.
     */
    getFirst(): E | undefined;
    /**
     * Time Complexity: O(1)
     * Space Complexity: O(1)
     */
    /**
     * Time Complexity: O(1)
     * Space Complexity: O(1)
     *
     * The `popLast()` function removes and returns the last element in a data structure.
     * @returns The value that was removed from the data structure.
     */
    popLast(): E | undefined;
    /**
     * Time Complexity: O(1)
     * Space Complexity: O(1)
     */
    /**
     * Time Complexity: O(1)
     * Space Complexity: O(1)
     *
     * The `getLast()` function returns the last element in an array-like data structure.
     * @returns The last element in the array "_nodes" is being returned.
     */
    getLast(): E | undefined;
    /**
     * Time Complexity: O(1)
     * Space Complexity: O(1)
     */
    /**
     * Time Complexity: O(1)
     * Space Complexity: O(1)
     *
     * The get function returns the element at the specified index in an array-like data structure.
     * @param {number} index - The index parameter is a number that represents the position of the element you want to
     * retrieve from the array.
     * @returns The element at the specified index in the `_nodes` array is being returned. If there is no element at that
     * index, `null` is returned.
     */
    get(index: number): NonNullable<E> | null;
    /**
     * The function checks if the size of a data structure is less than or equal to zero.
     * @returns The method is returning a boolean value indicating whether the size of the object is less than or equal to 0.
     */
    isEmpty(): boolean;
}
export declare class ArrayDeque<E> {
    protected _nodes: E[];
    get nodes(): E[];
    get size(): number;
    /**
     * Time Complexity: O(1)
     * Space Complexity: O(1)
     */
    /**
     * Time Complexity: O(1)
     * Space Complexity: O(1)
     *
     * The function "addLast" adds a value to the end of an array.
     * @param {E} value - The value parameter represents the value that you want to add to the end of the array.
     * @returns The return value is the new length of the array after the value has been added.
     */
    addLast(value: E): number;
    /**
     * Time Complexity: O(1)
     * Space Complexity: O(1)
     */
    /**
     * Time Complexity: O(1)
     * Space Complexity: O(1)
     *
     * The function "popLast" returns and removes the last element from an array, or returns null if the array is empty.
     * @returns The method `popLast()` returns the last element of the `_nodes` array, or `null` if the array is empty.
     */
    popLast(): E | null;
    /**
     * Time Complexity: O(n)
     * Space Complexity: O(1)
     */
    /**
     * Time Complexity: O(n)
     * Space Complexity: O(1)
     *
     * The `popFirst` function removes and returns the first element from an array, or returns null if the array is empty.
     * @returns The `popFirst()` function returns the first element of the `_nodes` array, or `null` if the array is
     * empty.
     */
    popFirst(): E | null;
    /**
     * Time Complexity: O(n)
     * Space Complexity: O(1)
     */
    /**
     * Time Complexity: O(n)
     * Space Complexity: O(1)
     *
     * The function "addFirst" adds a value to the beginning of an array.
     * @param {E} value - The value parameter represents the value that you want to add to the beginning of the array.
     * @returns The return value of the `addFirst` function is the new length of the array `_nodes` after adding the
     * `value` at the beginning.
     */
    addFirst(value: E): number;
    /**
     * Time Complexity: O(1)
     * Space Complexity: O(1)
     */
    /**
     * Time Complexity: O(1)
     * Space Complexity: O(1)
     *
     * The `getFirst` function returns the first element of an array or null if the array is empty.
     * @returns The function `getFirst()` is returning the first element (`E`) of the `_nodes` array. If the array is
     * empty, it will return `null`.
     */
    getFirst(): E | null;
    /**
     * Time Complexity: O(1)
     * Space Complexity: O(1)
     */
    /**
     * Time Complexity: O(1)
     * Space Complexity: O(1)
     *
     * The `getLast` function returns the last element of an array or null if the array is empty.
     * @returns The method `getLast()` returns the last element of the `_nodes` array, or `null` if the array is empty.
     */
    getLast(): E | null;
    /**
     * Time Complexity: O(1)
     * Space Complexity: O(1)
     */
    /**
     * Time Complexity: O(1)
     * Space Complexity: O(1)
     *
     * The get function returns the element at the specified index in an array, or null if the index is out of bounds.
     * @param {number} index - The index parameter is a number that represents the position of the element you want to
     * retrieve from the array.
     * @returns The method is returning the element at the specified index in the `_nodes` array. If the element exists, it
     * will be returned. If the element does not exist (i.e., the index is out of bounds), `null` will be returned.
     */
    get(index: number): E | null;
    /**
     * Time Complexity: O(1)
     * Space Complexity: O(1)
     */
    /**
     * Time Complexity: O(1)
     * Space Complexity: O(1)
     *
     * The set function assigns a value to a specific index in an array.
     * @param {number} index - The index parameter is a number that represents the position of the element in the array
     * that you want to set a new value for.
     * @param {E} value - The value parameter represents the new value that you want to set at the specified index in the
     * _nodes array.
     * @returns The value that is being set at the specified index in the `_nodes` array.
     */
    set(index: number, value: E): E;
    /**
     * Time Complexity: O(n)
     * Space Complexity: O(1)
     */
    /**
     * Time Complexity: O(n)
     * Space Complexity: O(1)
     *
     * The insert function adds a value at a specified index in an array.
     * @param {number} index - The index parameter specifies the position at which the value should be inserted in the
     * array. It is a number that represents the index of the array where the value should be inserted. The index starts
     * from 0, so the first element of the array has an index of 0, the second element has
     * @param {E} value - The value parameter represents the value that you want to insert into the array at the specified
     * index.
     * @returns The splice method returns an array containing the removed elements, if any. In this case, since no elements
     * are being removed, an empty array will be returned.
     */
    insert(index: number, value: E): E[];
    /**
     * Time Complexity: O(n)
     * Space Complexity: O(1)
     */
    /**
     * Time Complexity: O(n)
     * Space Complexity: O(1)
     *
     * The delete function removes an element from an array at a specified index.
     * @param {number} index - The index parameter specifies the position of the element to be removed from the array. It
     * is a number that represents the index of the element to be removed.
     * @returns The method is returning an array containing the removed element.
     */
    delete(index: number): E[];
    /**
     * The function checks if an array called "_nodes" is empty.
     * @returns The method `isEmpty()` is returning a boolean value. It returns `true` if the length of the `_nodes` array
     * is 0, indicating that the array is empty. Otherwise, it returns `false`.
     */
    isEmpty(): boolean;
}
